#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <io.h>
#include <fcntl.h>
#include "emmimp.h"
#include "moveext.h"

#ifndef __TURBOC__
#define asm _asm
#endif

static int for_386max = 0;
static int raw_dump = 0;

static unsigned long get_emm_imp_addr(int fd)
{
	unsigned char buf[6];
	int i;

	for (i = 0; i < 6; i++)
		buf[i] = 0;
	buf[0] = 1;
	asm lea dx, word ptr buf
	asm mov cx, 6
	asm mov bx, fd
	asm mov ax, 4402h
	asm int 21h
	asm jc fail
	printf("emm import structure address:%08lx\n", *(unsigned long *) buf);
	printf("emm import structure version:%02x%02x\n", buf[5], buf[4]);
	return *(unsigned long *) buf;
fail:
	printf("emm import struct address get failed\n");
	return 0;
}

static void (far *mode_switch(unsigned int ver))(void)
{
	void (far *entry)(void);
	void far *p;

	asm push ds
	asm push es
	asm mov dx, 0
	asm mov ax, 0
	asm mov es, ax
	asm mov ds, ax
	asm mov di, ver
	asm mov si, 0
	asm mov cx, 0
	asm mov bx, 0
	asm mov ax, 1605h
	asm int 2fh
	asm mov p, bx
	asm mov p + 2, es
	asm mov entry, si
	asm mov entry + 2, ds
	asm pop es
	asm pop ds
	return entry;
}

static void emulate_windows_exit(void)
{
	asm mov ax, 1606h
	asm int 2fh
}

static void switch_to_real(void (far *entry)(void))
{
	asm cli
	asm push ds
	asm push es
	asm push di
	asm push si
	asm push bp
	asm mov ax, 0
	asm call dword ptr entry;
	asm pop bp
	asm pop si
	asm pop di
	asm pop es
	asm pop ds
}

static void switch_to_prot(void (far *entry)(void))
{
	asm cli
	asm push ds
	asm push es
	asm push di
	asm push si
	asm push bp
	asm mov ax, 1
	asm call dword ptr entry;
	asm pop bp
	asm pop si
	asm pop di
	asm pop es
	asm pop ds

	asm sti
}


static void dump(unsigned char *p, int n, unsigned long base)
{
	int i;

	for (i = 0; i < n; i++) {
		if (i % 16 == 0)
			printf("%8lx: ", base + i);
		printf("%02x ", *p++);
		if (i % 16 == 15)
			printf("\n");
	}
}

static int detailed = 0;

static void *dump_hdr(void *p)
{
	wems_hdr_t *whp;
	wems_framestat_t *wfp;
	int i, j;

	whp = p;
	if (detailed) {
		printf("flag0:0x%x\n", whp->flag0);
		printf("resv0:0x%x\n", whp->resv0);
		printf("size:0x%x bytes\n", whp->size);
	}
	printf("version:0x%x\n", whp->version);
	if (detailed)
		printf("resv1:0x%x\n", whp->resv1);
	for (i = 0; i < 64; i++) {
		wfp = whp->frame + i;
		printf("frame[0x%02x] (%04x:0):", i, i * 0x400);
		if (!(wfp->type & 1)) {
			if (wfp->type & 0x3c) {
				printf("UMB");
				for (j = 0; j < 4; j++) {
					if (wfp->type & (1 << (j + 2))) {
						printf("/%04x",
							(i * 4 + j) * 0x100);
					}
				}
				printf("/umbinfo index:%d", wfp->handle);
			}
		} else {
			printf("%sEMS (phys page %02d)",
					(wfp->type & 2) ? "" : "large ",
					wfp->physpg_num);
			if (wfp->handle && wfp->handle != 0xff) {
				printf("/mapped to handle %d page %d",
					wfp->handle, wfp->logpage);
			}
		}
		printf("\n");
	}
	if (detailed)
		printf("resv2:0x%x\n", whp->resv2);
	p = whp + 1;
	return p;
}

static void *dump_umbinfo(void *p)
{
	unsigned char *cp;
	unsigned long *lp;
	int n, i;

	cp = p;
	n = *cp++;	/* # of umb-info */
	printf("# of umbinfo:%d\n", n);
	for (i = 0; i < n; i++) {
		lp = (unsigned long *) cp;
		printf("umbinfo[%2d]:%08lx %08lx %08lx %08lx\n",
			i, lp[0], lp[1], lp[2], lp[3]);
		cp += 16;
	}
	return cp;
}

static void *dump_ems_info(void *p)
{
	int i, n;
	unsigned char *cp;
	wems_handle_info_t *whip;
	unsigned char namebuf[9];

	cp = p;
	n = *cp++;
	whip = (wems_handle_info_t *) cp;
	namebuf[8] = 0;
	for (i = 0; i < n; i++, whip++) {
		memcpy(namebuf, whip->hndlname, 8);
		printf("EMS handle %2d: name=\"%s\", %d EMS pages, "
		       "pagemap at %08lx\n",
		       whip->handle, whip->hndlname, whip->pages,
		       whip->pgmap_phys_addr);
	}
	return whip;
}

static void *dump_other_info(void *p, int len)
{
	unsigned int *ip;
	unsigned char *cp;
	free_pg_entry_t *fp;
	int i, n;
	wems_xms_info_t *wxp;
	umb_free_seg_t *ufp;
	unsigned char namebuf[21];

	ip = p;
	printf("realmode int 67 vector:%04x:%04x\n", ip[1], ip[0]);
	ip += 2;
	printf("hma_page_table_addr:%08lx\n", *(unsigned long *) ip);
	ip += 2;
	cp = (unsigned char *) ip;

	printf("# of free page lists:%d\n", n = *cp);
	cp++;
	fp = (free_pg_entry_t *) cp;
	for (i = 0; i < n; i++, fp++) {
		printf("free page list[%02d]:page %08lx, %ld pages\n", 
			i, fp->physpgnum, fp->numpg);
	}
	cp = (unsigned char *) fp;

	printf("# of XMS info:%d\n", n = *cp);
	cp++;
	wxp = (wems_xms_info_t *) cp;
	for (i = 0; i < n; i++, wxp++) {
		printf("XMS handle %04x:Win %s, %ld KB, at %08lx\n",
			wxp->handle, (wxp->flag & 1) ? "usable" : "non-usable", 
			wxp->kb, wxp->baseaddr);
	}
	cp = (unsigned char *) wxp;

	printf("# of umb free seg:%d\n", n = *cp);
	cp++;
	ufp = (umb_free_seg_t *) cp;
	for (i = 0; i < n; i++, ufp++) {
		printf("umb free seg:%04x, 0x%x paragraphs\n",
			ufp->startseg, ufp->parasize);
	}
	cp = (unsigned char *) ufp;

	len -= cp - (unsigned char *) p;
	if (len <= 0)
		return cp;
	namebuf[20] = 0;
	memcpy(namebuf, cp, 20);
	printf("maker:\"%s\"\n", namebuf);
	cp += 20;
	memcpy(namebuf, cp, 20);
	printf("product:\"%s\"\n", namebuf);
	cp += 20;
	return cp;
}

static void dump_detailed(void *p, int n, unsigned long base)
{
	unsigned char *savep;

	savep = p;
	p = dump_hdr(p);
	p = dump_umbinfo(p);
	p = dump_ems_info(p);
	/* remaining length */
	n -= ((unsigned char *) p) - savep;
	if (n <= 0)
		return;
	dump_other_info(p, n);
}

static void dump_emm_imp(unsigned long addr, unsigned char *bufp)
{
	static unsigned char buf[0x1000];
	unsigned int n;


	if (!bufp) {
		move_ext_mem(addr, buf, sizeof buf);
		bufp = buf;
	}
	n = *(unsigned int *) (bufp + 2);
	printf("sizeof EMM import struct:0x%x bytes\n", n);
	if (raw_dump)
		dump(bufp, n, addr);
	else
		dump_detailed(bufp, n, addr);
}

static unsigned char buf[0x1000];
static unsigned int ver = 0x30a;	/* Windows version */

main(int ac, unsigned char **av)
{
	void (far *entry)(void);
	int i;
	int fd;
	unsigned long emm_imp_addr;

	if (ac > 1) {
		for (i = 1; i < ac; i++) {
			if (!strcmp(av[i], "-31"))
				ver = 0x30a;
			else if (!strcmp(av[i], "-30"))
				ver = 0x300;
			else if (!strcmp(av[i], "-386max"))
				for_386max = 1;
			else if (!strcmp(av[i], "-r"))
				raw_dump = 1;
			else if (!strcmp(av[i], "-d"))
				detailed = 1;
			else {
				printf("emmimp [-30] [-31] [-386max] [-r] [-d]\n"
				       "\tdumps the EMM import structure\n"
				       "\t-30:emulates win 3.0\n"
				       "\t-31:emulates win 3.1(default)\n"
				       "\t-386max:for 386max (PC only)\n"
				       "\t-r:raw dump\n"
				       "\t-d:detailed\n"
				       );
				exit(1);
			}
		}
	}
	if (!set_move_machine()) {
		printf("unknown machine\n");
		exit(1);
	}
	if (for_386max)
		set_move_to_386max_mode();

	fd = open("EMMXXXX0", O_RDONLY | O_BINARY);
	if (fd < 0)
		fd = open("$MMXXXX0", O_RDONLY | O_BINARY);
	if (fd < 0)	/* ??? */
		fd = open("EMMQXXX0", O_RDONLY | O_BINARY);
	if (fd < 0) {
		printf("can't open emm\n");
		exit(1);
	}
	emm_imp_addr = get_emm_imp_addr(fd);
	close(fd);
	if (!emm_imp_addr)
		return 1;
	entry = mode_switch(ver);
	printf("v86/prot switch entry:%08lx\n", entry);
	if (!entry) {
		printf("illegal entry\n");
		emulate_windows_exit();
		return 1;
	}
	if (for_386max) {
		switch_to_real(entry);
		move_ext_mem(emm_imp_addr, buf, sizeof buf);
		switch_to_prot(entry);
		dump_emm_imp(emm_imp_addr, buf);
	} else {
		switch_to_real(entry);
		switch_to_prot(entry);
		dump_emm_imp(emm_imp_addr, NULL);
	}
	emulate_windows_exit();
	return 0;
}
