#include <stdio.h>
#include <string.h>
#include <dos.h>
#include "moveext.h"

#ifndef __TURBOC__
#define asm _asm
#endif

typedef unsigned long uint32;	/* double word */
typedef unsigned int uint16;	/* word */
typedef unsigned char uint8;	/* byte */

#error define  BIOS_INT15_HANDLER below with BIOS int 15h vector address
/* if BIOS int15h handler is f000:f859, then
 * #define BIOS_INT15_HANDLER 0xf000f859L
 */
#define	BIOS_INT15_HANDLER 0xf000f859L

/*-----------------------------------------------------------------
 * ext mem -> conventional mem move (both for PC-AT & PC-9801)
 *-----------------------------------------------------------------*/

static int this_machine = 0;
static int for_386max = 0;

static int move_ext_mem_pc(unsigned long srcaddr, void far *dest,
		unsigned int bytes)
{
	unsigned long destaddr;
	void far *p;
	static unsigned char gdt[0x30] = {0};
	unsigned long bios_handler;

	bios_handler = BIOS_INT15_HANDLER;
	memset(gdt, 0, sizeof gdt);
	gdt[0x10] = bytes;
	gdt[0x11] = bytes >> 8;
	gdt[0x12] = srcaddr;
	gdt[0x13] = srcaddr >> 8;
	gdt[0x14] = srcaddr >> 16;
	gdt[0x15] = 0x93;

	destaddr = FP_SEG(dest);
	destaddr <<= 4;
	destaddr += FP_OFF(dest);
	gdt[0x18] = bytes;
	gdt[0x19] = bytes >> 8;
	gdt[0x1a] = destaddr;
	gdt[0x1b] = destaddr >> 8;
	gdt[0x1c] = destaddr >> 16;
	gdt[0x1d] = 0x93;
	p = &gdt;
	if (for_386max)
		goto _386max;
	asm les si, p
	asm mov ah, 87h
	asm mov cx, bytes
	asm inc cx
	asm shr cx, 1
	asm int 15h
	asm jmp short done
_386max:
	asm les si, p
	asm mov ah, 87h
	asm mov cx, bytes
	asm inc cx
	asm shr cx, 1
	/* call far to original int 15h vector */
	asm pushf
	asm call dword ptr bios_handler
done:
	/* error check omitted */
	return 1;
}

/* segment descriptor */
typedef struct {
	uint16 lim0_15;		/* Bits 0-15 of the segment limit */
	uint16 base0_15;		/* Bits 0-15 of the segment base */
	uint8 base16_23;		/* Bits 16-23 of the segment base */
	uint8 acc_byte;			/* Access byte */
	uint8 lim16_19;		/* Bits 16-19 of the segment limit
					   plus some flags */
	uint8 base24_31;		/* Bits 24-31 of the segment base */
} desc_t;

static int move_ext_mem_98(unsigned long srcaddr, void far *dest,
		unsigned int bytes)
{
	desc_t *p;
	desc_t far *fp;
	unsigned long destaddr;
	static desc_t desc[6] = {
		{0, 0, 0, 0, 0},	/* offset 0, NULL desc */
		{0, 0, 0, 0, 0},	/* offset 8h, created by BIOS */
		{0, 0, 0, 0, 0},	/* offset 10h, src desc */
		{0, 0, 0, 0, 0},	/* offset 18h, dest desc */
		{0, 0, 0, 0, 0},	/* offset 20h, created by BIOS */
		{0, 0, 0, 0, 0},	/* offset 28h, created by BIOS */
	};

	destaddr = FP_SEG(dest);
	destaddr <<= 4;
	destaddr += FP_OFF(dest);
	/* set src descriptor */
	p = desc + 2;	/* offset 10h */
	p->lim0_15 = 0xffff;
	p->base0_15 = srcaddr;
	p->base16_23 = (srcaddr >> 16) & 0xff;
	p->base24_31 = (srcaddr >> 24) & 0xff;
	p->acc_byte = 0x93;	/* data, expand up, r/w, dpl = 0 */

	/* set dest descriptor */
	p = desc + 3;	/* offset 18h */
	p->lim0_15 = 0xffff;
	p->base0_15 = destaddr;
	p->base16_23 = (destaddr >> 16) & 0xff;
	p->base24_31 = (destaddr >> 24) & 0xff;
	p->acc_byte = 0x93;	/* data, expand up, r/w, dpl = 0 */

	fp = desc;
	asm mov cx, bytes
	asm les bx, fp
	asm mov si, 0
	asm mov di, 0
	asm mov ah, 90h
	asm clc
	asm int 1fh
	asm jnc _OK
	return 0;
_OK:
	return 1;
}


static int machine_type(void)	/* returns: 0 for unknown, 1 for PC-AT, 2 for PC-9801 */
{
	unsigned int rc;
	unsigned char chval;

	rc = 0;	/* unknown */
	asm push ds
	asm push si
	asm push di
	/* do AT get realtime clock call */
	asm xor cx, cx
	asm mov ah, 4
	asm int 1ah
	asm mov chval, ch
	asm pop di
	asm pop si
	asm pop ds
	/* chval contains current century */
	if (chval == 0x19 || chval == 0x20)
		rc = 1;	/* PC-AT */
	else {
		asm push ds
		asm push si
		asm push di
		/* do PC98 get printer status */
		asm clc
		asm mov ah, 12h
		asm int 1ah
		asm pop di
		asm pop si
		asm pop ds
		asm jc	not_98
		rc = 2;	/* PC-98 */
not_98:;
	}
	return rc;
}

int set_move_machine(void)
{
	int machine;

	machine = machine_type();
	this_machine = machine;
	return machine;
}

void set_move_to_386max_mode(void)
{
	for_386max = 1;
}

void move_ext_mem(unsigned long srcaddr, void far *dest, unsigned int bytes)
{
	if (!this_machine) {
		set_move_machine();
		if (!this_machine)
			return;
	}
	if (this_machine == 2)	/* PC-9801 */
		move_ext_mem_98(srcaddr, dest, bytes);
	else	/* PC-AT */
		move_ext_mem_pc(srcaddr, dest, bytes);
}

